// menu.js
//==============================================================================
//     ____        ____
//    /    \____/ /    \          Author:  Jay Brockway
//    \/    |      |    \         Created: April 3, 2006
//          |      |____/         Revised: May 23, 2006
//          |      |    \         Version: 1.4.0 (version, update, build)
//          |      |     \        Email:   jay-javascript at vushta.com
//      ___ |      |_    /        Copyright (C) 2006-2006 by Jay Brockway
//     /   \| *   /  \__/ *       License: GNU General Public License
//
//==============================================================================
//    Program's Function:   Generates a HTML based table.
//==============================================================================
//    General Design:   The script display a hierarchical menu.
//==============================================================================
//    Copyright (C) 2006-2006 by Jay Brockway
//
//    GNU General Public License
//
//    This program is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program; if not, write to the Free Software
//    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
//==============================================================================

//==============================================================================
// Define a constructor for createTable objects.
function Menu()
{
    //--------------------------------------------------------------------------
    // Initialize object properties;
    // The data matrix array is a "table of contents" for all data matricies.
    // Columns are
    // Name, Type, Menu Title, URL Title, URL / Submenu name, Frame Location.
    // Example: ["menu",    "MENU", "Menu 1.2",         "MAIN1.1", "mainFrame"],
    this.menuItems;


    this.triangle;          // The name of the triangle indicating a sub-menu.
    this.defaultWidth;      // Default width for non-DOM compliant browsers.
    this.defaultTop;        // Default top position for non-DOM compliant browsers.
    this.hideTimeoutId;     // The ID returned from setTimeout() to hide the menu.

    //==========================================================================
    // Define methods for the object.
    this.showMenu = Menu_showMenu;
    this.hideMenu = Menu_hideMenu;
}


//==============================================================================
// Name:    showMenu
// Purpose: Display the menu items defined in the given array.
// Design:  Cycle through the menu array and construct a sub-menu containing all
//          array elements for this ID.
// Input:   left: the horizontal coordinate of the left edge.
//          top: the vertical coordinate of the top edge.
//          id: The ID of the new child node.
//          parentId: The ID of the parent node to attach the child node to.

// Open a second window for debugging.
// var win2 = window.open("", "window2" ,"toolbar=no,location=no,directories=no,status=no,menubar=no,resizeable=yes,scrollbars=yes,outerWidth=600,outerHeight=600,width=600,height=600,left=0,top=0");
// win2.document.open("text/html");
// Close this window to get a fresh one when this reloaded.  We can still write
// to this window.
// win2.document.close();

function Menu_showMenu(left, top, id, parentId)
{
    //--------------------------------------------------------------------------
    // Determine which browser we are running in since not all functions we
    // need are supported by all browsers.
    var browser = "Unknown";
    var userAgent = navigator.userAgent;

    // Test for Safari browser.
    var pattern = /Safari/g;
    if (userAgent.search(/Safari/g) >= 0)
    {
        browser = "Safari";
    }
    // Test for Mozilla browser.
    else if (userAgent.search(/MSIE/g) >= 0)
    {
        browser = "IE";
    }
    // Set to Mozilla browser by default.
    else
    {
        browser = "Mozilla";
    }
    // win2.document.writeln("userAgent: " + userAgent + "<br>");
    // win2.document.writeln("Browser: " + browser + "<br>");

    //--------------------------------------------------------------------------
    var parentNode = document.getElementById(parentId);

    // Create a local variable of this object.
    var my = this;

    //--------------------------------------------------------------------------
    // Remove other children from the parent menu before creating a new child.
    this.hideMenu(parentId);

    //--------------------------------------------------------------------------
    // Create the menu if it has not already been created.
    if(! document.getElementById(id))
    {
        // win2.document.writeln("Left:"+left+ "; Top:"+top+ "; ID:"+id+ "; Parent ID:"+parentId+ "<br>");
        var mainNode = document.createElement("div");
        mainNode.id = id;
        mainNode.style.position = "absolute";
        mainNode.style.left = left + "px";
        mainNode.style.top = top + "px";
        mainNode.style.margin = "0px 10px 0px 0px";
        mainNode.style.padding = "0px 0px 0px 0px";

        // Determine the width of mainNode and set the table node to the same width.
        var tableWidth = this.defaultWidth + "px";               // Set the default width.
        if (browser == "Mozilla")
        {
            try
            {
                tableWidth = document.defaultView.getComputedStyle(mainNode, "").getPropertyValue("width");
            }
            catch (e)
            {   /* Ignore the error. */ }
        }
        else if (browser == "Safari")
        {
            tableWidth = "100%";
        }
        // Table width is set at the bottom of this code section for IE.

        // Create the table of menu items.
        var table = document.createElement("div");
        table.className = "menuBox";

        for (var i=0; i < this.menuItems.length; i++)
        {
            if (this.menuItems[i][0] == id)
            {
                // Create each row in the menu.
                var item = document.createElement("div");
                item.className = "menuItem";
                item.title = this.menuItems[i][3];
                item.url = this.menuItems[i][4];
                item.target = this.menuItems[i][5];
                // Pass values to the new menu object.
                item.childId = this.menuItems[i][4];
                item.parentId = id;

                // Set the text value for the new link.
                item.appendChild(document.createTextNode(this.menuItems[i][2]));

                item.onmouseout =
                    function(event)
                    {
                        this.className = "menuItem";

                        if (my.hideTimeoutId != null)
                        {
                            // Clear any previous timout.
                            clearTimeout(my.hideTimeoutId);
                            // win2.document.writeln("Clear ID PREVIOUS: " + my.hideTimeoutId + "<br>");
                        }
                        // Call hideMainMenu() function after 1000 milliseconds.
                        my.hideTimeoutId = setTimeout("hideMainMenu()", 1000);
                        // win2.document.writeln("Set ID: " + my.hideTimeoutId + "<br>");
                    }


                // Check if this item is a LINK or MENU element type.
                //-- Link Item -------------------------------------------------
                if (this.menuItems[i][1] == "LINK")
                {
                    // Open the URL then close the menu.
                    item.onclick =
                        function(event)
                        {
                            open(this.url, this.target);

                            // Call hideMainMenu() function after 500 milliseconds.
                            my.hideTimeoutId = setTimeout("hideMainMenu()", 500);
                            // win2.document.writeln("Set ID: " + my.hideTimeoutId + "<br>");
                        }

                    item.onmouseover =
                        function(event)
                        {
                            this.className = "menuItemHover";
                            status = this.url;

                            // Cancel the menu hiding event.
                            clearTimeout(my.hideTimeoutId);
                            // win2.document.writeln("Clear ID LINK: " + my.hideTimeoutId + "<br>");

                            // Clears the other submenus.
                            my.showMenu (0, 0, this.childId, this.parentId);
                        }
                }
                //-- Menu Item -------------------------------------------------
                else if (this.menuItems[i][1] == "MENU")
                {
                    // Set the arrow image for the new link.
                    var image = document.createElement("img");
                    image.src = this.triangle;
                    image.className = "arrow";
                    item.arrow = image;
                    item.appendChild(image);

                    item.onmouseover =
                        function(event)
                        {
                            this.className = "menuItemHover";
                            status = this.url;

                            // Cancel the menu hiding event.
                            clearTimeout(my.hideTimeoutId);
                            // win2.document.writeln("Clear ID LINK: " + my.hideTimeoutId + "<br>");


                            // Set the default left edge coordinate.
                            var subMenuLeft = my.defaultWidth;
                            // Set the default top edge coordinate.
                            var subMenuTop = my.defaultTop;

                            // Check if CSS DOM standard is supported and set the
                            // dynamic left edge coordinate of the child sub-menu.
                            if (browser == "Mozilla")
                            {
                                try
                                {
                                    if (document.defaultView.getComputedStyle != null)
                                        subMenuLeft = document.defaultView.getComputedStyle(document.getElementById(this.parentId),null).getPropertyValue("width");
                                    subMenuTop = parseInt(document.defaultView.getComputedStyle(this.arrow,null).getPropertyValue("top"));
                                }
                                catch(e)
                                {   /* Ignore the error. */  }
                            }
                            else if (browser == "Safari")
                            {
                                try
                                {
                                    if (document.defaultView.getComputedStyle != null)
                                        subMenuLeft = document.defaultView.getComputedStyle(document.getElementById(this.parentId),null).getPropertyValue("width");
                                    subMenuTop = this.offsetTop;
                                }
                                catch(e)
                                {   /* Ignore the error. */  }
                            }
                            else if (browser == "IE")
                            {
                                subMenuLeft =  parseInt(document.getElementById(this.parentId).offsetWidth) - 3;
                                subMenuTop = this.offsetTop;
                            }

                            // Show submenu with 4 pixels of overlap and a 4 pixel drop.
                            my.showMenu (parseInt(subMenuLeft) - 4,
                                         parseInt(subMenuTop) + 4,
                                         this.childId,
                                         this.parentId
                                        );
                        }
                }
                table.appendChild(item);
            }
        }

        mainNode.appendChild(table);
        parentNode.appendChild(mainNode);

        // Determine the width of mainNode and set the table node to the same width.
        // IE does not know the width of the node until it is fully specified.
        if (browser == "IE")
        {
            tableWidth = mainNode.offsetWidth + "px";
        }
        table.style.width = tableWidth;

        // win2.document.writeln("tableWidth: " + tableWidth + "<br>");
    }
}

//==============================================================================
// Remove the sub-menu nodes from the given parent node.
function Menu_hideMenu(id)
{
    // win2.document.writeln("Remove children from ID: " + id + "<br>");
    var node = document.getElementById(id);

    for (var i=0; i < this.menuItems.length; i++)
    {
        // win2.document.writeln("Removing child with ID: " + this.menuItems[i][3] + "<br>");
        try
        {
            if ((node) && (node.hasChildNodes()))
                node.removeChild(document.getElementById(this.menuItems[i][4]));
        }
        catch(e)
        {   /* Ignore the error. */ }
    }
}

//==============================================================================
// Remove all the menu nodes.
function hideMainMenu()
{
    var node = document.getElementById('menu');

    try
    {
        if (node.hasChildNodes())
        {
            // Remove all children nodes.
            while (node.hasChildNodes())
            {
                node.removeChild(node.lastChild);
            }
        }
    }
    catch(e)
    {   /* Ignore the error. */ }
}
